//
// Fragment shader for PCF with dynamic number samples Shadow Maps
// Based on code from GPU Gems II
//

#define	EPS		0.4
#define	fSize	0.2

uniform sampler2D		mainMap;
uniform sampler2DShadow	shadowMap;
uniform	sampler2DRect 	samplesMap;

void main (void)
{
	vec3 	offs [4];
	vec4	clr  = texture2D ( mainMap, gl_TexCoord [0].xy );
	vec4	tex  = gl_TexCoord [1] / gl_TexCoord [1].w;
	float	sh   = shadow2D ( shadowMap, tex.xyz ).x;
	
	offs [0] = vec3 ( 0.02,  0.0,  0.0 );
	offs [1] = vec3 ( -0.01, 0.01, 0.0 );
	offs [2] = vec3 ( 0.0,  -0.02, 0.0 );
	offs [3] = vec3 ( -0.01, -0.01, 0.0 );
		
	for ( int i = 0; i < 4; i++ )
		sh += shadow2D ( shadowMap, tex.xyz + fSize * offs [i] ).x;
		
	sh *= 0.2;
	
	if ( sh * (1.0 - sh) > EPS )			// we're in a half-shadow, use 32 samples
	{
		float	sum = 0.0;
		
		for ( int i = 0; i < 16; i++ )
		{
			vec4	pt = texture2DRect ( samplesMap, vec2 ( i, 0.0 ) );
			
			sum += shadow2D ( shadowMap, tex.xyz + fSize * 0.015 * vec3 ( pt.xy, 0.0 ) ).x;
			sum += shadow2D ( shadowMap, tex.xyz + fSize * 0.015 * vec3 ( pt.zw, 0.0 ) ).x;
		}
		
		sh = (sum + sh * 5.0) / (32.0 + 5.0);
	}

	gl_FragColor = clr *  sh;
}
